@@ -1,7 +1,7 @@
 /*
  * vmfs-tools - Tools to access VMFS filesystems
  * Copyright (C) 2009 Christophe Fillot <cf@utc.fr>
- * Copyright (C) 2009 Mike Hommey <mh@glandium.org>
+ * Copyright (C) 2009,2011 Mike Hommey <mh@glandium.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -70,6 +70,8 @@ static int vmfs_inode_read(vmfs_inode_t *inode,const u_char *buf)
 
    if (inode->type == VMFS_FILE_TYPE_RDM) {
       inode->rdm_id = read_le32(buf,VMFS_INODE_OFS_RDM_ID);
+   } else if (inode->zla == VMFS5_ZLA_BASE + VMFS_BLK_TYPE_FD) {
+      memcpy(inode->content, buf + VMFS_INODE_OFS_CONTENT, inode->size);
    } else {
       for(i=0;i<VMFS_INODE_BLK_COUNT;i++)
          inode->blocks[i] = vmfs_inode_read_blk_id(buf,i);
@@ -125,7 +127,7 @@ int vmfs_inode_update(const vmfs_inode_t *inode,int update_blk_list)
       buf_len -= VMFS_INODE_BLK_COUNT * sizeof(uint32_t);
    }
 
-   if (vmfs_lvm_write(inode->fs->lvm,inode->mdh.pos,buf,buf_len) != buf_len)
+   if (vmfs_device_write(inode->fs->dev,inode->mdh.pos,buf,buf_len) != buf_len)
       return(-1);
 
    return(0);
@@ -279,13 +281,24 @@ int vmfs_inode_get_block(const vmfs_inode_t *inode,off_t pos,uint32_t *blk_id)
 {
    const vmfs_fs_t *fs = inode->fs;
    u_int blk_index;
+   uint32_t zla;
+   int vmfs5_extension;
 
    *blk_id = 0;
 
    if (!inode->blk_size)
       return(-EIO);
 
-   switch(inode->zla) {
+   /* This doesn't make much sense but looks like how it's being coded. At
+    * least, the result has some sense. */
+   zla = inode->zla;
+   if (zla >= VMFS5_ZLA_BASE) {
+      vmfs5_extension = 1;
+      zla -= VMFS5_ZLA_BASE;
+   } else
+      vmfs5_extension = 0;
+
+   switch(zla) {
       case VMFS_BLK_TYPE_FB:
       case VMFS_BLK_TYPE_SB:
          blk_index = pos / inode->blk_size;
@@ -328,6 +341,11 @@ int vmfs_inode_get_block(const vmfs_inode_t *inode,off_t pos,uint32_t *blk_id)
          break;
       }
 
+      case VMFS_BLK_TYPE_FD:
+         if (vmfs5_extension) {
+            *blk_id = inode->id;
+            break;
+         }
       default:
          /* Unexpected ZLA type */
          return(-EIO);
@@ -671,35 +689,6 @@ int vmfs_inode_truncate(vmfs_inode_t *inode,off_t new_len)
    return(0);
 }
 
-/* Show block list of an inode */
-void vmfs_inode_show_blocks(const vmfs_inode_t *inode)
-{
-   const vmfs_fs_t *fs = inode->fs;
-   uint64_t blk_size;
-   uint32_t blk_per_pb;
-   u_int blk_count;
-   int i;
-
-   blk_size = inode->blk_size;
-   
-   if (!blk_size)
-      return;
-
-   blk_count = (inode->size + blk_size - 1) / blk_size;
-
-   if (inode->zla == VMFS_BLK_TYPE_PB) {
-      blk_per_pb = fs->pbc->bmh.data_size / sizeof(uint32_t);
-      blk_count = (blk_count + blk_per_pb - 1) / blk_per_pb;
-   }
-
-   for(i=0;i<blk_count;i++) {
-      if (i && !(i % 4)) printf("\n");
-      printf("0x%8.8x ",inode->blocks[i]);
-   }
-
-   printf("\n");
-}
-
 /* Call a function for each allocated block of an inode */
 int vmfs_inode_foreach_block(const vmfs_inode_t *inode,
                              vmfs_inode_foreach_block_cbk_t cbk,
@@ -711,9 +700,8 @@ int vmfs_inode_foreach_block(const vmfs_inode_t *inode,
    uint32_t blk_id;
    u_int blk_total;
    u_int blk_count;
-   int i,j,err_count;
+   int i,j;
 
-   err_count = 0;
    blk_total = 0;
    blk_per_pb = 0;
 
@@ -771,39 +759,6 @@ int vmfs_inode_foreach_block(const vmfs_inode_t *inode,
    return(0);
 }
 
-/* Check that an inode block is allocated */
-static void vmfs_inode_check_block_alloc(const vmfs_inode_t *inode,
-                                         uint32_t pb_blk,
-                                         uint32_t blk_id,
-                                         void *opt_arg)
-{
-   int *err_count = opt_arg;
-
-   if (vmfs_block_get_status(inode->fs,blk_id) <= 0) {
-      if (!pb_blk)
-         fprintf(stderr,"Block 0x%8.8x is not allocated\n",blk_id);
-      else
-         fprintf(stderr,"Block 0x%8.8x in PB 0x%8.8x is not allocated\n",
-                 blk_id,pb_blk);
-
-      (*err_count)++;
-   }
-}
-
-/* Check that all blocks bound to an inode are allocated */
-int vmfs_inode_check_blocks(const vmfs_inode_t *inode)
-{
-   int status,err_count = 0;
-
-   status = vmfs_inode_foreach_block(inode,vmfs_inode_check_block_alloc,
-                                     &err_count);
-
-   if (status == -1)
-      return(-1);
-
-   return(err_count);
-}
-
 /* Get inode status */
 int vmfs_inode_stat(const vmfs_inode_t *inode,struct stat *buf)
 {
@@ -816,6 +771,8 @@ int vmfs_inode_stat(const vmfs_inode_t *inode,struct stat *buf)
    buf->st_atime = inode->atime;
    buf->st_mtime = inode->mtime;
    buf->st_ctime = inode->ctime;
+   buf->st_blksize = M_BLK_SIZE;
+   buf->st_blocks  = inode->blk_count * (inode->blk_size / S_BLKSIZE);
    return(0);
 }
 
